@@ -222,12 +222,12 @@ class Loader:
             (is_attrs, _attrload),
             (is_any, _anyload),
             (is_newtype, _newtypeload),
-        ]  # type: List[Tuple[Callable[[Any], bool], Callable[[Loader, Any, Type], Any]]]
+        ]  # type: List[Tuple[Callable[[Any], bool], Callable[[Loader, Any, Any], Any]]]
 
         for k, v in kwargs.items():
             setattr(self, k, v)
 
-        self._indexcache = {}  # type: Dict[Type, int]
+        self._indexcache = {}  # type: Dict[Any, Callable[[Loader, Any, Any], Any]]
 
         self._unionload_discriminatorcache = {}  # type: Dict[Type, Tuple[Optional[str], Optional[Dict[Any, Type]]]]
 
@@ -262,14 +262,13 @@ class Loader:
         It is only needed when calling load recursively from
         a custom handler.
         """
-        p_index = self._indexcache.get(type_)
+        cached_f = self._indexcache.get(type_)
 
-        if p_index is not None:
-            index = p_index
+        if cached_f is not None:
+            func = cached_f
         else:
             try:
                 index = self.index(type_)
-                self._indexcache[type_] = index
             except ValueError:
                 raise TypedloadTypeError(
                     'Cannot deal with value of type %s' % tname(type_),
@@ -277,14 +276,14 @@ class Loader:
                     type_=type_
                 )
 
+            func = self._indexcache[type_] = self.handlers[index][1]
+
             # Add type to known types, to resolve ForwardRef later on
             if self.frefs is not None and hasattr(type_, '__name__'):
                 typename = type_.__name__
                 if typename not in self.frefs:
                     self.frefs[typename] = type_
 
-        func = self.handlers[index][1]
-
         try:
             return func(self, value, type_)
         except Exception as e:
@@ -293,7 +292,7 @@ class Loader:
             raise e
 
 
-def _forwardrefload(l: Loader, value: Any, type_: type) -> Any:
+def _forwardrefload(l: Loader, value: Any, type_) -> Any:
     """
     This resolves a ForwardRef.
 
@@ -313,11 +312,11 @@ def _forwardrefload(l: Loader, value: Any, type_: type) -> Any:
     return l.load(value, t, annotation=Annotation(AnnotationType.FORWARDREF, tname))
 
 
-def _anyload(l: Loader, value: Any, type_: type) -> Any:
+def _anyload(l: Loader, value: Any, type_) -> Any:
     return value
 
 
-def _literalload(l: Loader, value: Any, type_: type) -> Any:
+def _literalload(l: Loader, value: Any, type_) -> Any:
     """
     Checks if the value is within the allowed literals and
     returns it.
@@ -327,7 +326,7 @@ def _literalload(l: Loader, value: Any, type_: type) -> Any:
     raise TypedloadValueError('Not one of the allowed values in %s' % tname(type_), value=value, type_=type_)
 
 
-def _basicload(l: Loader, value: Any, type_: type) -> Any:
+def _basicload(l: Loader, value: Any, type_) -> Any:
     """
     This converts a value into a basic type.
 
@@ -352,7 +351,7 @@ def _basicload(l: Loader, value: Any, type_: type) -> Any:
     return value
 
 
-def _dictload(l: Loader, value, type_) -> Dict:
+def _dictload(l: Loader, value: Any, type_) -> Dict:
     """
     This loads into something like Dict[str,str]
 
@@ -360,8 +359,44 @@ def _dictload(l: Loader, value, type_) -> Dict:
     """
     key_type, value_type = type_.__args__
 
+    key_handler = l._indexcache.get(key_type)
+    if key_handler is not None:
+        key_f = key_handler
+    else:
+        try:
+            key_f = l._indexcache[key_type] = l.handlers[l.index(key_type)][1]
+        except ValueError:
+            raise TypedloadValueError(
+                'Cannot deal with value of type %s (key of %s)' % (tname(key_type), tname(type_)),
+                value=value,
+                type_=key_type
+            )
+
+    # Same thing for the value
+    value_handler = l._indexcache.get(value_type)
+    if value_handler is not None:
+        value_f = value_handler
+    else:
+        try:
+            key_f = l._indexcache[value_type] = l.handlers[l.index(value_type)][1]
+        except ValueError:
+            raise TypedloadValueError(
+                'Cannot deal with value of type %s (value of %s)' % (tname(value_type), tname(type_)),
+                value=value,
+                type_=value_type
+            )
+
     value = _dictequivalence(l, value)
 
+    # Try fast load
+    try:
+        return {
+            key_f(l, k, key_type): value_f(l, v, value_type) for k, v in value.items()
+        }
+    except Exception:
+        # Failed, do the slow method with exception tracking
+        pass
+
     try:
         return {
             l.load(k, key_type, annotation=Annotation(AnnotationType.KEY, k)): l.load(v, value_type, annotation=Annotation(AnnotationType.VALUE, v))
@@ -370,12 +405,10 @@ def _dictload(l: Loader, value, type_) -> Dict:
         raise TypedloadAttributeError(str(e), type_=type_, value=value)
 
 
-def _tupleload(l: Loader, value, type_) -> Tuple:
+def _tupleload(l: Loader, value: Any, type_) -> Tuple:
     """
     This loads into something like Tuple[int,str]
     """
-    if isinstance(value, dict):
-        raise TypedloadTypeError('Unable to load dictionary as a tuple', value=value, type_=type_)
     if HAS_TUPLEARGS:
         args = type_.__args__
     else:
@@ -384,6 +417,8 @@ def _tupleload(l: Loader, value, type_) -> Tuple:
     if len(args) == 2 and args[1] == ...: # Tuple[something, ...]
         return _iterload(l, value, type_, tuple)
     else: # Tuple[something, something, somethingelse]
+        if isinstance(value, dict):
+            raise TypedloadTypeError('Unable to load dictionary as a tuple', value=value, type_=type_)
         if l.failonextra and len(value) > len(args):
             raise TypedloadValueError('Value is too long for type %s' % tname(type_), value=value, type_=type_)
         elif len(value) < len(args):
@@ -453,7 +488,7 @@ def _dataclassload(l: Loader, value: Dict[str, Any], type_) -> Any:
     return _objloader(l, fields, necessary_fields, type_hints, value, type_)
 
 
-def _dictequivalence(l: Loader, value: Any) -> Dict:
+def _dictequivalence(l: Loader, value: Any) -> Any:
     '''
     Helper function to convert classes that are functionally
     the same as a dict into a dict.
@@ -469,7 +504,7 @@ def _dictequivalence(l: Loader, value: Any) -> Dict:
     return value
 
 
-def _objloader(l: Loader, fields: Set[str], necessary_fields: Set[str], type_hints, value: Dict[str, Any], type_) -> Any:
+def _objloader(l: Loader, fields: Set[str], necessary_fields: Set[str], type_hints, value: Any, type_) -> Any:
     '''
     Helper function to load dict-like data into an object.
     '''
@@ -484,7 +519,7 @@ def _objloader(l: Loader, fields: Set[str], necessary_fields: Set[str], type_hin
             value = newvalue
             vfields = set(value.keys())
 
-    if necessary_fields.intersection(vfields) != necessary_fields:
+    if len(necessary_fields.intersection(vfields)) != len(necessary_fields):
         raise TypedloadValueError(
             'Value does not contain fields: %s which are necessary for type %s' % (
                 necessary_fields.difference(vfields),
@@ -520,7 +555,7 @@ def _objloader(l: Loader, fields: Set[str], necessary_fields: Set[str], type_hin
         raise TypedloadValueError(e)
 
 
-def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Any:
+def _namedtupleload(l: Loader, value: Any, type_) -> Any:
     """
     This loads a Dict[str, Any] into a NamedTuple.
     """
@@ -535,7 +570,7 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Any:
     return _objloader(l, fields, necessary_fields, type_hints, value, type_)
 
 
-def _typeddictload(l: Loader, value: Dict[str, Any], type_) -> Any:
+def _typeddictload(l: Loader, value: Any, type_) -> Any:
     """
     This loads a Dict[str, Any] into a NamedTuple.
     """
@@ -563,7 +598,7 @@ def _typeddictload(l: Loader, value: Dict[str, Any], type_) -> Any:
     return _objloader(l, fields, necessary_fields, type_hints, value, type_)
 
 
-def _unionload(l: Loader, value, type_) -> Any:
+def _unionload(l: Loader, value: Any, type_) -> Any:
     """
     Loads a value into a union.
 
@@ -654,7 +689,7 @@ def _unionload(l: Loader, value, type_) -> Any:
         )
 
 
-def _enumload(l: Loader, value, type_) -> Enum:
+def _enumload(l: Loader, value: Any, type_) -> Enum:
     """
     This loads something into an Enum.
 
@@ -692,7 +727,7 @@ def _enumload(l: Loader, value, type_) -> Enum:
     )
 
 
-def _noneload(l: Loader, value, type_) -> None:
+def _noneload(l: Loader, value: Any, type_) -> None:
     """
     Loads a value that can only be None,
     so it fails if it isn't
@@ -702,14 +737,14 @@ def _noneload(l: Loader, value, type_) -> None:
     raise TypedloadValueError('Not None', value=value, type_=type_)
 
 
-def _datetimeload(l: Loader, value, type_) -> Union[datetime.date, datetime.time, datetime.datetime]:
+def _datetimeload(l: Loader, value: Any, type_) -> Union[datetime.date, datetime.time, datetime.datetime]:
     try:
         return type_(*value)
     except TypeError as e:
         raise TypedloadTypeError(str(e), type_=type_, value=value)
 
 
-def _attrload(l, value, type_):
+def _attrload(l: Loader, value: Any, type_) -> Any:
     from attr._make import _Nothing as NOTHING
 
     fields = {i.name for i in type_.__attrs_attrs__}
@@ -746,14 +781,20 @@ def _strconstructload(l: Loader, value, type_):
     except TypeError as e:
         raise TypedloadTypeError(str(e), type_=type_, value=value)
     except Exception as e:
-        raise TypedloadException(str(e), type_=type, value=value)
+        raise TypedloadException(str(e), type_=type_, value=value)
 
 
-def _newtypeload(l: Loader, value, type_):
+def _newtypeload(l: Loader, value: Any, type_) -> Any:
     return l.load(value, type_.__supertype__)
 
 
-def _iterload(l: Loader, value, type_, function):
+def _iterload(l: Loader, value: Any, type_, function) -> Any:
+    """
+    Generic code to load iterables.
+
+    function is for example list, tuple, set. The call to
+    generate the destination type from an iterable.
+    """
     if isinstance(value, dict):
         raise TypedloadTypeError('Unable to load dictionary as an iterable', value=value, type_=type_)
     t = type_.__args__[0]
@@ -762,14 +803,23 @@ def _iterload(l: Loader, value, type_, function):
     fast = hasattr(value, '__getitem__')
 
     if fast:
-        try:
-            f = l.handlers[l.index(t)][1]
-        except ValueError:
-            raise TypedloadTypeError(
-                'Cannot deal with value of type %s' % tname(type_),
-                value=value,
-                type_=type_
-            )
+
+        # Get function pointer for the handler
+        cached_f = l._indexcache.get(t)
+
+        if cached_f:
+            f = cached_f
+        else:
+            try:
+                f = l._indexcache[t] = l.handlers[l.index(t)][1]
+            except ValueError:
+                raise TypedloadTypeError(
+                    'Cannot deal with value of type %s' % tname(type_),
+                    value=value,
+                    type_=type_
+                )
+
+        # Use the handler
         try:
             # Hopeful load calling the handler directly, skipping load()
             return function(f(l, v, t) for v in value)
